%
% function X = BlockDWT(x, block_sizes)
%
%   This function performs wavelet-domain block-based sampling of the
%   spatial-domain image x. block_sizes gives the size of blocks in
%   each level of the DWT.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%


function X = BlockDWT(x, block_sizes)

num_levels = length(block_sizes);

[num_rows num_cols] = size(x);

x = waveletcdf97(x, num_levels);

subband_num_rows = num_rows / 2^num_levels;
subband_num_cols = num_cols / 2^num_levels;

X{1} = x(1:subband_num_rows, 1:subband_num_cols);

for level = 1:num_levels
  
  block_size = block_sizes(level);
  
  subband1 = x(1:subband_num_rows, ...
      (subband_num_cols + 1):(2 * subband_num_cols));
  X{level + 1}{1} = im2col(subband1, [block_size block_size], 'distinct');
  subband2 = x((subband_num_rows + 1):(2 * subband_num_rows), ...
      1:subband_num_cols);
  X{level + 1}{2} = im2col(subband2, [block_size block_size], 'distinct');
  subband3 = x((subband_num_rows + 1):(2 * subband_num_rows), ...
      (subband_num_cols + 1):(2 * subband_num_cols));
  X{level + 1}{3} = im2col(subband3, [block_size block_size], 'distinct');

  subband_num_rows = subband_num_rows * 2;
  subband_num_cols = subband_num_cols * 2;
end
