%
% function x = InverseBlockDWT(X)
%
%   This function reconstructs the image x from its wavelet-domain
%   blocks given by X.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function x = InverseBlockDWT(X)

num_levels = length(X) - 1;

[baseband_num_rows baseband_num_cols] = size(X{1});

num_rows = baseband_num_rows * 2^num_levels;
num_cols = baseband_num_cols * 2^num_levels;

x = zeros(num_rows, num_cols);

x(1:baseband_num_rows, 1:baseband_num_cols) = X{1};

subband_num_rows = baseband_num_rows;
subband_num_cols = baseband_num_cols;

for level = 1:num_levels

  block_size = sqrt(size(X{level + 1}{1}, 1));
  
  x(1:subband_num_rows, ...
      (subband_num_cols + 1):(2 * subband_num_cols)) = ...
      col2im(X{level + 1}{1}, [block_size block_size], ...
      [subband_num_rows subband_num_cols], 'distinct');
  x((subband_num_rows + 1):(2 * subband_num_rows), ...
      1:subband_num_cols) = ...
      col2im(X{level + 1}{2}, [block_size block_size], ...
      [subband_num_rows subband_num_cols], 'distinct');
  x((subband_num_rows + 1):(2 * subband_num_rows), ...
      (subband_num_cols + 1):(2 * subband_num_cols)) = ...
      col2im(X{level + 1}{3}, [block_size block_size], ...
      [subband_num_rows subband_num_cols], 'distinct');
  
  subband_num_rows = subband_num_rows * 2;
  subband_num_cols = subband_num_cols * 2;
end

x = waveletcdf97(x, -num_levels);
