% 
% function y = MS_BCS_SPL_Encoder(current_image, Phi)
% 
%   This function performs BCS projections of each block of
%   in the DWT of the current image. The length of Phi (a cell array) gives
%   the number of levels of the DWT.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function y = MS_BCS_SPL_Encoder(current_image, Phi)

num_levels = length(Phi);

if (isnumeric(Phi{1}))
  for level = 1:num_levels
    block_sizes(level) = sqrt(size(Phi{level}, 2));
  end
else
  for level = 1:num_levels
    block_sizes(level) = Phi{level}{1};
  end
end

X = BlockDWT(current_image, block_sizes);

y{1} = X{1};

if (isnumeric(Phi{1}))
  for level = 1:num_levels
    for subband = 1:3
      y{level + 1}{subband} = Phi{level} * X{level + 1}{subband};
    end
  end
else
  for level = 1:num_levels
    for subband = 1:3
      y{level + 1}{subband} = Phi{level}{2}(X{level + 1}{subband});
    end
  end
end
