%
% function [ subrates, actual_subrate ] = ...
%     MS_BCS_SPL_GetSubrates(target_subrate, num_levels)
%
%   This function generates the subrates for each level of a DWT for
%   wavelet-domain block-based sampling with a overall target subrate
%   of target_subrate.
% 

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function [ subrates, actual_subrate ] = ...
    MS_BCS_SPL_GetSubrates(target_subrate, num_levels)

level = num_levels:-1:1;

weights = 16 .^ level;

level_factor = 3 ./ (2 .^ (2 * level));

baseband_factor = 2 .^ (-2 * num_levels);

for level = 1:num_levels
  prior_level_factor = 0;
  for prior_level = 1:(level - 1)
    prior_level_factor = prior_level_factor + ...
	level_factor(prior_level) * subrates(prior_level);
  end
  r = (target_subrate - baseband_factor - prior_level_factor) ./ ...
      (level_factor(level:num_levels) * weights(level:num_levels)');
  
  subrates(level) = min(weights(level) * r, 1);
  
end

actual_subrate = baseband_factor + level_factor * subrates';
