% 
% function y = MS_GPSR_Encoder(current_image, Phi)
% 
%   This function performs BCS projections of each level in the DWT of
%   the current image. The length of Phi (a cell array) gives
%   the number of levels of the DWT.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function y = MS_GPSR_Encoder(current_image, Phi)

num_levels = length(Phi);
[num_rows num_cols] = size(current_image);

if (isnumeric(Phi{1}))
  for level = 1:num_levels
    Phi_matrix = Phi{level};
    Phi{level} = @(x) Phi_matrix * x;
  end
elseif (iscell(Phi{1}))
  for level = 1:num_levels
    Phi{level} = Phi{level}{1};
  end
else
  disp('Incorrect Phi format');
  return;
end

X = waveletcdf97(current_image, num_levels);

subband_num_rows = num_rows / 2^num_levels;
subband_num_cols = num_cols / 2^num_levels;

y{1} = X(1:subband_num_rows, 1:subband_num_cols);

for level = 1:num_levels
  
  subband1 = X(1:subband_num_rows, ...
      (subband_num_cols + 1):(2 * subband_num_cols));
  subband2 = X((subband_num_rows + 1):(2 * subband_num_rows), ...
      1:subband_num_cols);
  subband3 = X((subband_num_rows + 1):(2 * subband_num_rows), ...
      (subband_num_cols + 1):(2 * subband_num_cols));

  y{level + 1} = Phi{level}([subband1(:) ; subband2(:) ; subband3(:)]);
  
  subband_num_rows = subband_num_rows * 2;
  subband_num_cols = subband_num_cols * 2;
end

