% 
% function run_experiment_bcsspl()
% 
%   This function runs the experiments for BCS-SPL in Table 2 of
%     J. E. Fowler, S. Mun, and E. W. Tramel, "Multiscale Block Compressed
%     Sensing with Smoothed Projected Landweber Reconstruction," in
%     Proceedings of the European Signal Processing Conference, Barcelona,
%     Spain, August 2011, pp. 564-568.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function run_experiment_bcsspl()

addpath('../BCS-SPL-1.2-1/');
addpath('../GPSR/');
addpath(genpath('../../l1magic/'));
addpath(genpath('../fast_cs_SRM/'));
addpath('../WaveletSoftware/');
addpath('../waveletcdf97/');
addpath('Images');

filename = 'lenna';
% filename = 'barbara';
% filename = 'goldhill';

subrates = [0.1 0.2 0.3 0.4 0.5];

num_trials = 1;

block_size = 32;

num_levels = 3;

original_filename = [filename '.pgm'];
original_image = double(imread(original_filename));
[num_rows num_cols] = size(original_image);
original_mean = mean(original_image(:));
X = original_image - original_mean;

for i = 1:length(subrates)
  subrate = subrates(i);

  disp(['Subrate = ' num2str(subrate)]);
  
  for trial = 1:num_trials
    projection_matrix_file = ['projections.bcsspl.' ...
	  num2str(block_size) '.' num2str(trial) '.mat'];
    
    Phi = BCS_SPL_GenerateProjection(block_size, subrate, ...
	projection_matrix_file);
    
    y = BCS_SPL_Encoder(X, Phi);
    
    tic
    reconstructed_image = BCS_SPL_DDWT_Decoder(y, Phi, num_rows, ...
	num_cols, num_levels);
    toc
    
    reconstructed_image = reconstructed_image + original_mean;
    
    imwrite(reconstructed_image/255, ...
	[filename '.bcsspl-ddwt.' num2str(subrate*100) '.pgm'], 'pgm');
    
    psnr(trial) = PSNR(original_image, reconstructed_image);
  end
  
  psnr = mean(psnr);
  
  disp(['BCS-SPL-DDWT PSNR = ' num2str(psnr) ' dB']);
end
