% 
% function run_experiment_msgpsr()
% 
%   This function runs the experiments for MS-GPSR in Table 2 of
%     J. E. Fowler, S. Mun, and E. W. Tramel, "Multiscale Block Compressed
%     Sensing with Smoothed Projected Landweber Reconstruction," in
%     Proceedings of the European Signal Processing Conference, Barcelona,
%     Spain, August 2011, pp. 564-568.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function run_experiment_msgpsr()

addpath('../BCS-SPL-1.2-1/');
addpath('../GPSR/');
addpath(genpath('../../l1magic/'));
addpath('../fast_cs_SRM/Transforms/');
addpath('../fast_cs_SRM/Measurements/');
addpath('../WaveletSoftware/');
addpath('../waveletcdf97/');
addpath('Images');

filename = 'lenna';
% filename = 'barbara';
% filename = 'goldhill';

target_subrates = [0.1 0.2 0.3 0.4 0.5];

num_trials = 1;

num_levels = 3;

projection_type = 'dct';
% projection_type = 'gaussian';

original_filename = [filename '.pgm'];
original_image = double(imread(original_filename));
[num_rows num_cols] = size(original_image);
original_mean = mean(original_image(:));
X = original_image - original_mean;

for level = 1:num_levels
  subband_sizes{level} = [num_rows num_cols] / 2^(num_levels - ...
      level + 1);
end
  
for i = 1:length(target_subrates)
  target_subrate = target_subrates(i);
  
  [ subrates, actual_subrate ] = ...
      MS_BCS_SPL_GetSubrates(target_subrate, num_levels); 
  disp(['Target subrate = ' num2str(target_subrate)]);
  
  for trial = 1:num_trials
    projection_file = ['projections.msgpsr.' projection_type ...
	  '.' num2str(num_levels) ...
	  '.' num2str(trial) '.mat'];
    Phi = MS_GPSR_GenerateProjection(subband_sizes, subrates, ...
	projection_file, projection_type);
    
    y = MS_GPSR_Encoder(X, Phi);
    
    cnt = numel(y{1});
    for level = 1:num_levels
      cnt = cnt + numel(y{level + 1});
    end
    disp(['Actual subrate = ' num2str(cnt / num_rows / num_cols)]);
    
    tic
    reconstructed_image = MS_GPSR_Decoder(y, Phi, ...
	subrates, num_rows, num_cols);
    toc
    
    reconstructed_image = reconstructed_image + original_mean;
    
    imwrite(reconstructed_image/255, ...
	[filename '.msgpsr.' num2str(target_subrate*100) ...
	  '.pgm'], 'pgm');
    
    psnr(trial) = PSNR(original_image, reconstructed_image);
  end
  
  psnr = mean(psnr);
  
  disp(['MS-GPSR PSNR = ' num2str(psnr) ' dB']);
end  
