% 
% function run_experiment_tv()
% 
%   This function runs the experiments for TV in Table 2 of
%     J. E. Fowler, S. Mun, and E. W. Tramel, "Multiscale Block Compressed
%     Sensing with Smoothed Projected Landweber Reconstruction," in
%     Proceedings of the European Signal Processing Conference, Barcelona,
%     Spain, August 2011, pp. 564-568.
%

%
% MS-BCS-SPL: Multiscale Block Compressed Sensing - Smooth Projected Landweber
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function run_experiment_tv()

addpath('../BCS-SPL-1.2-1/');
addpath('../GPSR/');
addpath(genpath('../../l1magic/'));
addpath(genpath('../fast_cs_SRM/'));
addpath('../WaveletSoftware/');
addpath('../waveletcdf97/');
addpath('Images');

filename = 'lenna';
% filename = 'barbara';
% filename = 'goldhill';

subrates = [0.1 0.2 0.3 0.4 0.5];

original_filename = [filename '.pgm'];
original_image = double(imread(original_filename));
[num_rows num_cols] = size(original_image);

X = original_image;
minX = min(X(:));
maxX = max(X(:));
X = (X - minX)/(maxX - minX);
X = X(:);

for i = 1:length(subrates)
  subrate = subrates(i);

  disp(['Subrate = ' num2str(subrate)]);
  
  % Generate projection matrix Phi, N x M matrix
  M = length(X);
  N = round(subrate * M);
  rand_vect = randperm(M)';
  select_vect = randperm(M);
  select_vect = select_vect(1:N);
  trans_mode = 'BWHT';
  blk_size = 32;
  Phi = @(z) blk_f1d(z, select_vect, rand_vect, trans_mode, blk_size);
  Phi_T = @(z) blk_t1d(z, M, select_vect, rand_vect,trans_mode, blk_size); 
  
  y = Phi(X);
  
  epsilon = 0.01;
  
  x0 = Phi_T(y);
  tic
  tv_image = tvqc_logbarrier(x0, Phi, Phi_T, y, ...
      epsilon, 1e-3, 5, 1e-8, 200);
  toc
  
  tv_image = reshape((maxX - minX)*tv_image + minX, ...
      size(original_image));
  tv_psnr = PSNR(original_image, tv_image);
  
  disp(['TV PSNR = ' num2str(tv_psnr) ' dB']);
  imwrite(tv_image/255, ...
      [filename '.tv.' num2str(subrate * 100) '.pgm'], 'pgm');
end
